from __future__ import annotations  # for python 3.8


class Version:
    as_str: str = None
    as_ags4_str: str = None
    as_int: int = None
    family: str = None
    family_as_int: int = None
    major: str = None
    minor: str = None
    improv: str = None
    patch: str = None
